/*****************************************************************/
/*                                                               */
/* Written by Martin van den Nieuwelaar, martin@gadgets.co.nz    */
/*                                                               */
/* This header is free when distributed with the bbnfc software. */
/* In all other cases it is copyright (c) 2001 Gadgets           */
/* Software & Professional Services Ltd.                         */
/*                                                               */
/*****************************************************************/

/* Header size in bytes */
#define NETFLOW_V1_HEADER_LENGTH 16
#define NETFLOW_V5_HEADER_LENGTH 24
#define NETFLOW_V7_HEADER_LENGTH 24
#define NETFLOW_V8_HEADER_LENGTH 28

/* The smallest header we might receive */
#define NETFLOW_MINIMUM_HEADER_LENGTH 16

/* Flow size in bytes */
#define NETFLOW_V1_FLOW_LENGTH 48
#define NETFLOW_V5_FLOW_LENGTH 48
#define NETFLOW_V7_FLOW_LENGTH 52
#define NETFLOW_V8_AS_FLOW_LENGTH 28
#define NETFLOW_V8_PROTOPORT_FLOW_LENGTH 28
#define NETFLOW_V8_DSTPREFIX_FLOW_LENGTH 32
#define NETFLOW_V8_SRCPREFIX_FLOW_LENGTH 32
#define NETFLOW_V8_PREFIX_FLOW_LENGTH 40

/* Maximum number of flows per export */
#define NETFLOW_V1_MAX_FLOWS 24
#define NETFLOW_V5_MAX_FLOWS 30
#define NETFLOW_V7_MAX_FLOWS 27
/* Some aggregation schemes will allow fewer than 51 flows, but this */
/* is a good starting point even though not always entirely accurate */
#define NETFLOW_V8_MAX_FLOWS 51

/* Aggregation schemes */
#define V8_AS_FLOW_AGGREGATION 1
#define V8_PROTOPORT_FLOW_AGGREGATION 2
#define V8_SRCPREFIX_FLOW_AGGREGATION 3
#define V8_DSTPREFIX_FLOW_AGGREGATION 4
#define V8_PREFIX_FLOW_AGGREGATION 5


/* The two structures that describe the contents of a version 1 Netflow */
/* Export.  The header of 16 bytes in size is followed by a number of */
/* flow entries each of 48 bytes in length. */

struct netflow_export_v1_header {
  uint16_t version;
  uint16_t count;
  uint32_t SysUptime;
  uint32_t unix_secs;
  uint32_t unix_nsecs;
};

struct netflow_export_v1_flow {
  struct in_addr srcaddr;
  struct in_addr dstaddr;
  struct in_addr nexthop;
  uint16_t input;
  uint16_t output;
  uint32_t dPkts;
  uint32_t dOctets;
  uint32_t First;
  uint32_t Last;
  uint16_t srcport;
  uint16_t dstport;
  uint16_t pad1;
  unsigned char prot;
  unsigned char tos;
  unsigned char tcp_flags;
  unsigned char pad2, pad3, pad4;
  uint32_t reserved;
};


/* The two structures that describe the contents of a version 5 Netflow */
/* Export.  The header of 24 bytes in size is followed by a number of */
/* flow entries each of 48 bytes in length. */

struct netflow_export_v5_header {
  uint16_t version;
  uint16_t count;
  uint32_t SysUptime;
  uint32_t unix_secs;
  uint32_t unix_nsecs;
  uint32_t flow_sequence;
  unsigned char engine_type;
  unsigned char engine_id;
  uint16_t reserved;
};

struct netflow_export_v5_flow {
  struct in_addr srcaddr;
  struct in_addr dstaddr;
  struct in_addr nexthop;
  uint16_t input;
  uint16_t output;
  uint32_t dPkts;
  uint32_t dOctets;
  uint32_t First;
  uint32_t Last;
  uint16_t srcport;
  uint16_t dstport;
  unsigned char pad1;
  unsigned char tcp_flags;
  unsigned char prot;
  unsigned char tos;
  uint16_t src_as;
  uint16_t dst_as;
  unsigned char src_mask;
  unsigned char dst_mask;
  uint16_t pad2;
};


/* The two structures that describe the contents of a version 7 Netflow */
/* Export.  The header of 24 bytes in size is followed by a number of */
/* flow entries each of 52 bytes in length. */

struct netflow_export_v7_header {
  uint16_t version;
  uint16_t count;
  uint32_t SysUptime;
  uint32_t unix_secs;
  uint32_t unix_nsecs;
  uint32_t flow_sequence;
  uint32_t reserved;
};

struct netflow_export_v7_flow {
  struct in_addr srcaddr;
  struct in_addr dstaddr;
  struct in_addr nexthop;
  uint16_t input;
  uint16_t output;
  uint32_t dPkts;
  uint32_t dOctets;
  uint32_t First;
  uint32_t Last;
  uint16_t srcport;
  uint16_t dstport;
  unsigned char field_flags;
  unsigned char tcp_flags;
  unsigned char prot;
  unsigned char tos;
  uint16_t src_as;
  uint16_t dst_as;
  unsigned char src_mask;
  unsigned char dst_mask;
  uint16_t flow_flags;
  struct in_addr router_sc;
};

/* The six structures that describe the contents of a version 8 Netflow */
/* Export.  The header of 28 bytes in size is followed by a number of */
/* flow entries, the exact size of each depending on the aggregation */
/* scheme used. */

struct netflow_export_v8_header {
  uint16_t version;
  uint16_t count;
  uint32_t SysUptime;
  uint32_t unix_secs;
  uint32_t unix_nsecs;
  uint32_t flow_sequence;
  unsigned char engine_type;
  unsigned char engine_id;
  unsigned char aggregation;
  unsigned char agg_version;
  uint16_t sampling_interval;
  uint16_t reserved;
};

struct netflow_export_v8_as_flow {
  uint32_t flows;
  uint32_t dPkts;
  uint32_t dOctets;
  uint32_t First;
  uint32_t Last;
  uint16_t src_as;
  uint16_t dst_as;
  uint16_t input;
  uint16_t output;
};

struct netflow_export_v8_protoport_flow {
  uint32_t flows;
  uint32_t dPkts;
  uint32_t dOctets;
  uint32_t First;
  uint32_t Last;
  unsigned char prot;
  unsigned char pad;
  uint16_t reserved;
  uint16_t srcport;
  uint16_t dstport;
};

struct netflow_export_v8_dstprefix_flow {
  uint32_t flows;
  uint32_t dPkts;
  uint32_t dOctets;
  uint32_t First;
  uint32_t Last;
  struct in_addr dst_prefix;
  unsigned char dst_mask;
  unsigned char pad;
  uint16_t dst_as;
  uint16_t output;
  uint16_t reserved;
};

struct netflow_export_v8_srcprefix_flow {
  uint32_t flows;
  uint32_t dPkts;
  uint32_t dOctets;
  uint32_t First;
  uint32_t Last;
  struct in_addr src_prefix;
  unsigned char src_mask;
  unsigned char pad;
  uint16_t src_as;
  uint16_t input;
  uint16_t reserved;
};

struct netflow_export_v8_prefix_flow {
  uint32_t flows;
  uint32_t dPkts;
  uint32_t dOctets;
  uint32_t First;
  uint32_t Last;
  struct in_addr src_prefix;
  struct in_addr dst_prefix;
  unsigned char src_mask;
  unsigned char dst_mask;
  uint16_t reserved;
  uint16_t src_as;
  uint16_t dst_as;
  uint16_t input;
  uint16_t output;
};
